#define FD_SETSIZE 1
#include "channel.h"
#include <algorithm>
#include "ltr\\include\\ltrapi.h"
#include <stdio.h>

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
TChannel::TChannel()
    : sock(INVALID_SOCKET)
    {
    default_timeout = LTR_DEFAULT_SEND_RECV_TIMEOUT;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TChannel::Close()
    {
    if (sock != INVALID_SOCKET)
        {
        shutdown(sock, SD_BOTH);
        closesocket(sock);
        sock = INVALID_SOCKET;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TChannel::Open(DWORD addr, WORD port, DWORD timeout_ms)
    {
    Close();

    SOCKET s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (s == INVALID_SOCKET)
        return false;

    //    non-blocking mode
    ULONG nonblocking = 1;
    if (SOCKET_ERROR == ioctlsocket(s, FIONBIO, &nonblocking))
        return false;

    sockaddr_in peer;
    ZeroMemory(&peer, sizeof(peer));
    peer.sin_family = AF_INET;
    peer.sin_port = htons(port);
    peer.sin_addr.s_addr = htonl(addr);

    fd_set fd_w, fd_e;

    bool connected = false;
    SetTimer(timeout_ms);
    while (!connected)
        {
        if (SOCKET_ERROR == connect(s, (sockaddr*)&peer, sizeof(peer)))
            {
            if (WSAEWOULDBLOCK != WSAGetLastError())
                break;
            FD_ZERO(&fd_w);
            FD_SET(s, &fd_w);
            FD_ZERO(&fd_e);
            FD_SET(s, &fd_e);
            if (select(1, NULL, &fd_w, &fd_e, timer.TimeLeftSelect()) < 1)
                break;
            if (FD_ISSET(s, &fd_w))
                {
                connected = true;
                }
            else if (FD_ISSET(s, &fd_e))
                {
                int err, optlen = sizeof(err);
                if (SOCKET_ERROR == getsockopt(s, SOL_SOCKET, SO_ERROR, (char*)&err, &optlen))
                    break;
                //   ,  
                if ((WSAECONNREFUSED != err) && (WSAENETUNREACH != err) &&
                    (WSAEHOSTUNREACH != err) && (WSAETIMEDOUT != err))
                    break;
                Sleep(50);
                }
            }
        else
            {
            connected = true;
            }
        } /* while (!connected) */

    if (!connected)
        {
        Close();
        }
    else
        {
        sock = s;
        }
    return connected;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
INT TChannel::Recv(LPBYTE buf, DWORD size)
    {
    if (!buf) return LTR_ERROR_PARAMETERS;
    if (!IsOpen())
        {
        Sleep(1);
        return LTR_ERROR_CHANNEL_CLOSED;
        }

    DWORD offset = 0;
    bool timed_out = false;
    bool error = false;
    fd_set fd_r;

    while (!error && !timed_out && (offset < size))
        {
        FD_ZERO(&fd_r);
        FD_SET(sock, &fd_r);
        switch (select(1, &fd_r, NULL, NULL, timer.TimeLeftSelect()))
            {
            case SOCKET_ERROR:
                error = true;
                break;
            case 0: // 
                timed_out = true;
                break;
            default: //  
                int res = recv(sock, buf + offset, size - offset, 0);
                if (SOCKET_ERROR == res)
                    {
                    error = (WSAEWOULDBLOCK != WSAGetLastError());
                    }
                else if (0 == res) //  
                    {
                    error = true;
                    }
                else
                    {
                    offset += res;
                    }
                break;
            }
        }

    return (error) ? LTR_ERROR_RECV : offset;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
INT TChannel::Send(LPBYTE buf, DWORD size)
    {
    if (!buf) return LTR_ERROR_PARAMETERS;
    if (!IsOpen())
        {
        Sleep(1);
        return LTR_ERROR_CHANNEL_CLOSED;
        }

    if (!size) return 0;

    DWORD offset = 0;
    bool timed_out = false;
    bool error = false;
    fd_set fd_w;

    while (!error && !timed_out && (offset < size))
        {
        //      
        int res = send(sock, buf + offset, size - offset, 0);
        if (SOCKET_ERROR == res)
            {
            if (WSAEWOULDBLOCK == WSAGetLastError())
                { //    
                FD_ZERO(&fd_w);
                FD_SET(sock, &fd_w);
                switch (select(1, NULL, &fd_w, NULL, timer.TimeLeftSelect()))
                    {
                    case SOCKET_ERROR:
                        error = true;
                        break;
                    case 0: // 
                        timed_out = true;
                        break;
                    }
                }
            else
                {
                error = true;
                }
            }
        else
            { // no error
            offset += res;
            }
        }

    return (error) ? LTR_ERROR_SEND : offset;
    }
/*------------------------------------------------------------------------------------------------*/

#if EXPORT_SEND_RECV
//      DLL.     .
/*------------------------------------------------------------------------------------------------*/
INT TChannel::Recv(LPBYTE buf, DWORD size, DWORD timeout_ms)
    {
    SetTimer(timeout_ms);
    return Recv(buf, size);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
INT TChannel::Send(LPBYTE buf, DWORD size, DWORD timeout_ms)
    {
    SetTimer(timeout_ms);
    return Send(buf, size);
    }
/*------------------------------------------------------------------------------------------------*/
#endif
/*================================================================================================*/
