#ifndef __CHANNEL__
#define __CHANNEL__
#include <Winsock2.h>

#define EXPORT_SEND_RECV    0

/*================================================================================================*/
class TChannelTimer
    {
private:
    DWORD timeout;
    DWORD start_time;
    TIMEVAL timeout_select;
public:
    TChannelTimer(DWORD timeout_ms = INFINITE)
        {
        timeout = timeout_ms;
        }

    void Start(DWORD timeout_ms)
        {
        start_time = GetTickCount();
        timeout = timeout_ms;
        }

    DWORD TimeLeft(void) const
        {
        if (INFINITE == timeout)
            return INFINITE;
        DWORD elapsed = GetTickCount() - start_time;
        return (elapsed < timeout) ? timeout - elapsed : 0;
        }

    const TIMEVAL* TimeLeftSelect(void)
        {
        if (INFINITE == timeout)
            return NULL;
        DWORD elapsed = GetTickCount() - start_time;
        DWORD left = (elapsed < timeout) ? timeout - elapsed : 0;
        timeout_select.tv_sec = left / 1000;
        timeout_select.tv_usec = (left % 1000) * 1000;
        return &timeout_select;
        }
    };
/*================================================================================================*/

/*================================================================================================*/
class TChannel
    {
private:
    SOCKET sock;
    DWORD default_timeout;
    TChannelTimer timer;
public:
    void SetDefaultTimeout(DWORD timeout_ms)
        {
        default_timeout = timeout_ms;
        }

    bool Open(DWORD addr, WORD port, DWORD timeout_ms);

    void Close();

    bool IsOpen() const
        {
        return (sock != INVALID_SOCKET);
        }

    void SetTimer(DWORD timeout_ms)
        {
        timer.Start(timeout_ms);
        }

    void SetTimerDefault()
        {
        timer.Start(default_timeout);
        }

    bool TimerExpired(void) const
        {
        return (0 == timer.TimeLeft());
        }

    //   Recv()  Send()       SetTimer()
    INT Recv(LPBYTE buf, DWORD size);
    INT Send(LPBYTE buf, DWORD size);

#if EXPORT_SEND_RECV
    //      DLL.     .
    INT __export Recv(LPBYTE buf, DWORD size, DWORD timeout_ms);
    INT __export Send(LPBYTE buf, DWORD size, DWORD timeout_ms);
#endif

    TChannel();
    ~TChannel()
        {
        Close();
        }
    };
/*================================================================================================*/
#endif
