#include "channel.h"
#include "ltr\\include\\ltr010api.h"
#include <stdio.h>

#define LTR010_COMMAND_TIMEOUT  1000

static const T_ERROR_STRING_DEF ErrorStrings[] = {
{ LTR010_ERROR_GET_ARRAY,       "   GET_ARRAY." },
{ LTR010_ERROR_PUT_ARRAY,       "   PUT_ARRAY." },
{ LTR010_ERROR_GET_MODULE_NAME, "   GET_MODULE_NAME." },
{ LTR010_ERROR_GET_MODULE_DESCR,"   GET_MODULE_DESCRIPTOR." },
{ LTR010_ERROR_INIT_FPGA,       "   INIT_FPGA." },
{ LTR010_ERROR_RESET_FPGA,      "   RESET_FPGA." },
{ LTR010_ERROR_INIT_DMAC,       "   INIT_DMAC." },
{ LTR_ERROR_LOAD_FPGA,          "   LOAD_FPGA." },
{ LTR010_ERROR_OPEN_FILE,       "  ." },
{ LTR010_ERROR_GET_INFO_FPGA,   "   GET_INFO_FPGA." }
};

//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Init(TLTR010 *module)
    {
    if (!module) return LTR_ERROR_PARAMETERS;
    return LTR_Init(&module->ltr);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Open(TLTR010 *module, DWORD saddr, WORD sport, CHAR *csn)
    {
    if (!module || !csn) return LTR_ERROR_PARAMETERS;
    INT res = LTR010_Close(module);
    if (res == LTR_OK)
        {
        module->ltr.saddr = saddr;
        module->ltr.sport = sport;
        memcpy(module->ltr.csn, csn, SERIAL_NUMBER_SIZE);
        module->ltr.cc = CC_CONTROL;
        res = LTR_Open(&module->ltr);
        }
    return res;
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Close(TLTR010 *module)
    {
    if (!module) return LTR_ERROR_PARAMETERS;
    return LTR_Close(&module->ltr);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_GetArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
    {
    if (!module || !buf) return LTR_ERROR_PARAMETERS;
    DWORD sbuf[] =
        {
        CONTROL_COMMAND_START, CONTROL_COMMAND_GET_ARRAY,
        address, size
        };
    return LTR__GenericCtlFunc(&module->ltr,
        sbuf, sizeof(sbuf), buf, size,
        LTR010_ERROR_GET_ARRAY, LTR010_COMMAND_TIMEOUT);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_PutArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
    {
    if (!module || !buf) return LTR_ERROR_PARAMETERS;
    size_t send_size = 4 * sizeof(DWORD) + size;
    LPBYTE pbuf = new BYTE[send_size];
    if (!pbuf) return LTR_ERROR_MEMORY_ALLOC;
    LPDWORD header = (LPDWORD)pbuf;
    header[0] = CONTROL_COMMAND_START;
    header[1] = CONTROL_COMMAND_PUT_ARRAY;
    header[2] = address;
    header[3] = size;
    memcpy(&header[4], buf, size);
    INT res = LTR__GenericCtlFunc(&module->ltr,
        pbuf, send_size, NULL, 0,
        LTR010_ERROR_PUT_ARRAY, LTR010_COMMAND_TIMEOUT);
    delete[] pbuf;
    return res;
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_GetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
    {
    if (!module) return LTR_ERROR_PARAMETERS;
    DWORD sbuf[] =
        {
        CONTROL_COMMAND_START, CONTROL_COMMAND_GET_DESCRIPTION
        };
    return LTR__GenericCtlFunc(&module->ltr,
        sbuf, sizeof(sbuf), description, sizeof(TDESCRIPTION_LTR010),
        LTR010_ERROR_GET_MODULE_DESCR, LTR010_COMMAND_TIMEOUT);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_LoadFPGA(TLTR010 *module, CHAR *fname, BYTE rdma, BYTE wdma)
    {
    if (!module || !fname) return LTR_ERROR_PARAMETERS;
    INT res = LTR_IsOpened(&module->ltr);
    if (res != LTR_OK) return res;

    FILE *f = fopen(fname, "rt");
    if (!f) return LTR010_ERROR_OPEN_FILE;

    BYTE data[8 + 2 + EP1K50_SIZE];
    *((LPDWORD)&data[0]) = CONTROL_COMMAND_START;
    *((LPDWORD)&data[4]) = CONTROL_COMMAND_LOAD_FPGA;
    data[8] = rdma;
    data[9] = wdma;
    DWORD datasize = 10;
    ZeroMemory(&data[datasize], sizeof(data) - datasize);
    while (!feof(f) && (datasize < sizeof(data)))
        {
        int d;
        fscanf(f, "%3d,", &d);
        data[datasize++] = d & 0xFF;
        }
    fclose(f);

    return LTR__GenericCtlFunc(&module->ltr,
        data, sizeof(data), NULL, 0,
        LTR_ERROR_LOAD_FPGA, 8000);
    }
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
LPCSTR WINAPI __export LTR010_GetErrorString(INT error)
    {
    for (size_t i = 0; i < sizeof(ErrorStrings) / sizeof(ErrorStrings[0]); i++)
        {
        if (ErrorStrings[i].code == error)
            return ErrorStrings[i].message;
        }
    return LTR_GetErrorString(error);
    }
//-----------------------------------------------------------------------------
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
    {
    return 1;
    }

