//---------------------------------------------------------------------------

#include <windows.h>
#include "channel.h"
#include "ltr\\include\\ltrbootapi.h"

#define LTRBOOTAPI_TIMEOUT  2000

static const T_ERROR_STRING_DEF ErrorStrings[] = {
{ LTRBOOT_ERROR_GET_ARRAY,          "   GET_ARRAY." },
{ LTRBOOT_ERROR_PUT_ARRAY,          "   PUT_ARRAY." },
{ LTRBOOT_ERROR_CALL_APPL,          "   CALL_APPLICATION." },
{ LTRBOOT_ERROR_GET_DESCRIPTION,    "   GET_DESCRIPTION." },
{ LTRBOOT_ERROR_PUT_DESCRIPTION,    "   PUT_DESCRIPTION." }
};

//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Init(TLTR010 *module)
    {
    return LTR010_Init(module);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Open(TLTR010 *module, DWORD saddr, WORD sport, CHAR *csn)
    {
    return LTR010_Open(module, saddr, sport, csn);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Close(TLTR010 *module)
    {
    return LTR010_Close(module);
    }
//-----------------------------------------------------------------------------
//                  
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_GetArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
    {
    if (!module || !buf) return LTR_ERROR_PARAMETERS;
    DWORD sbuf[] =
        {
        CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_GET_ARRAY,
        address, size
        };
    return LTR__GenericCtlFunc(&module->ltr,
        sbuf, sizeof(sbuf), buf, size,
        LTRBOOT_ERROR_GET_ARRAY, LTRBOOTAPI_TIMEOUT);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_PutArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
    {
    if (!module || !buf) return LTR_ERROR_PARAMETERS;
    size_t send_size = 4 * sizeof(DWORD) + size;
    LPBYTE pbuf = new BYTE[send_size];
    if (!pbuf) return LTR_ERROR_MEMORY_ALLOC;
    LPDWORD header = (LPDWORD)pbuf;
    header[0] = CONTROL_COMMAND_START;
    header[1] = CONTROL_COMMAND_BOOT_LOADER_PUT_ARRAY;
    header[2] = address;
    header[3] = size;
    memcpy(&header[4], buf, size);
    INT res = LTR__GenericCtlFunc(&module->ltr,
        pbuf, send_size, NULL, 0,
        LTRBOOT_ERROR_PUT_ARRAY, LTRBOOTAPI_TIMEOUT);
    delete[] pbuf;
    return res;
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_GetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
    {
    if (!module) return LTR_ERROR_PARAMETERS;
    DWORD sbuf[] =
        {
        CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_GET_DESCRIPTION
        };
    return LTR__GenericCtlFunc(&module->ltr,
        sbuf, sizeof(sbuf), description, sizeof(TDESCRIPTION_LTR010),
        LTRBOOT_ERROR_GET_DESCRIPTION, LTRBOOTAPI_TIMEOUT);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_SetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
    {
    const size_t send_size = 2 * sizeof(DWORD) + sizeof(TDESCRIPTION_LTR010);
    if (!module || !description) return LTR_ERROR_PARAMETERS;
    BYTE pbuf[send_size];
    LPDWORD header = (LPDWORD)pbuf;
    header[0] = CONTROL_COMMAND_START;
    header[1] = CONTROL_COMMAND_BOOT_LOADER_SET_DESCRIPTION;
    memcpy(&header[2], description, sizeof(TDESCRIPTION_LTR010));
    return LTR__GenericCtlFunc(&module->ltr,
        pbuf, send_size, NULL, 0,
        LTRBOOT_ERROR_PUT_DESCRIPTION, LTRBOOTAPI_TIMEOUT);
    }
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_CallApplication(TLTR010 *module, DWORD address)
    {
    if (!module) return LTR_ERROR_PARAMETERS;
    DWORD sbuf[] =
        {
        CONTROL_COMMAND_START, CONTROL_COMMAND_CALL_APPLICATION,
        address
        };
    return LTR__GenericCtlFunc(&module->ltr,
        sbuf, sizeof(sbuf), NULL, 0,
        LTRBOOT_ERROR_CALL_APPL, LTRBOOTAPI_TIMEOUT);
    }
//-----------------------------------------------------------------------------
LPCSTR WINAPI LTRBOOT_GetErrorString(INT error)
    {
    for (size_t i = 0; i < sizeof(ErrorStrings) / sizeof(ErrorStrings[0]); i++)
        {
        if (ErrorStrings[i].code == error)
            return ErrorStrings[i].message;
        }
    return LTR_GetErrorString(error);
    }
//-----------------------------------------------------------------------------

#pragma argsused
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fwdreason, LPVOID lpvReserved)
    {
    return 1;
    }
//---------------------------------------------------------------------------
